import 'dart:convert';

import 'package:dio/dio.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter_blue_plus/flutter_blue_plus.dart';
import 'package:flutter_blue_plus/gen/flutterblueplus.pb.dart' as photo;
import 'package:smart_utils/LocalStore.dart';
import 'package:smart_utils/http.dart';
import 'package:smart_utils/test/testData.dart';
import 'package:smart_utils/tools/EscPosByJson.dart';

BluetoothDevice getBluetoothDevice(String name, String address, int type) {
  return BluetoothDevice.fromProto(
    photo.BluetoothDevice(
      name: name,
      remoteId: address,
      type: photo.BluetoothDevice_Type.valueOf(type),
    ),
  );
}



class BlueToothPrint {

  static final FlutterBluePlus _blueTooth = FlutterBluePlus.instance;

  static const String blueLocalStorePrefix = 'BlueTooth';


  static Future<List> scan(int time) async {
    await _blueTooth.startScan(
      scanMode: const ScanMode(2),
      timeout: Duration(seconds: time),
    );
    final List<BluetoothDevice> resultDevices = [];
    _blueTooth.scanResults.listen((List<ScanResult> scanResults) {
      for (final ScanResult scanResult in scanResults) {
        resultDevices.add(scanResult.device);
      }
    });

    await _blueTooth.stopScan();

    List devices = resultDevices.toSet().toList().map((BluetoothDevice bluetoothDevice) => ({
      'address': bluetoothDevice.id.id,
      'name': bluetoothDevice.name,
      'type': bluetoothDevice.type.index,
    })).toList();
    return devices;
  }

  // 检查蓝牙是否已经链接
  static Future<BluetoothDevice?> getDevicesConnectedByAddress(String address) async {
    final List<BluetoothDevice> connectedDevices = await _blueTooth.connectedDevices;
    final int deviceConnectedIndex = connectedDevices.indexWhere((BluetoothDevice bluetoothDevice) {
      return bluetoothDevice.id.id == address;
    });
    if (deviceConnectedIndex == -1) return null;
    return connectedDevices[deviceConnectedIndex];
  }

  static Future<BluetoothDevice> getBluetoothDeviceByAlias(String alias) async {
    String? contentStr = await LocalStore.getItem('${blueLocalStorePrefix}_$alias');
    if (contentStr == null) return Future.error('蓝牙未连接');
    Map blueMap = jsonDecode(contentStr);
    return getBluetoothDevice(blueMap['name'], blueMap['address'], blueMap['type']);
  }



  static Future connect(String name, String address, int type, String alias) async {
    debugPrint('name: $name');
    debugPrint('address: $address');
    debugPrint('type: $type');
    BluetoothDevice? connectBlueDevice = await getDevicesConnectedByAddress(address);
    if (connectBlueDevice != null) {
      debugPrint('蓝牙已经链接');
      final endMtu = await connectBlueDevice.requestMtu(503);
      debugPrint('endMtu $endMtu');
      try {
        await LocalStore.setItem('${blueLocalStorePrefix}_$alias', jsonEncode({ 'name': name, 'address': address, 'type': type, 'alias': alias }));
        await LocalStore.setInt(address, endMtu);
      } catch (e) {
        debugPrint(e.toString());
      }
      return true;
    }
    final bluetoothDevice = getBluetoothDevice(name, address, type);
    debugPrint(bluetoothDevice.toString());
    await bluetoothDevice.connect(timeout: const Duration(seconds: 2));
    debugPrint('交换 mtu 值');
    final endMtu = await bluetoothDevice.requestMtu(503);
    await LocalStore.setItem('${blueLocalStorePrefix}_$alias', jsonEncode({ 'name': name, 'address': address, 'type': type, 'alias': alias }));
    await LocalStore.setInt(address, endMtu);
    return true;
  }


  static disconnect(String name, String address, int type, String alias) async {

    BluetoothDevice? connectBlueDevice = await getDevicesConnectedByAddress(address);
    if (connectBlueDevice == null) {
      return true;
    }


    connectBlueDevice.disconnect();

    await LocalStore.removeItem('${blueLocalStorePrefix}_$alias');
    await LocalStore.removeItem(address);

    return true;
  }

  static Future<Map<String, dynamic>?> getConnectedDeviceByAlias(String alias) async {
    try {
      BluetoothDevice? bluetoothDevice = await getBluetoothDeviceByAlias(alias);
      bluetoothDevice = await getDevicesConnectedByAddress(bluetoothDevice.id.id);
      if (bluetoothDevice == null) return null;
      return {
        'name': bluetoothDevice.name,
        'address': bluetoothDevice.id.id,
        'type': bluetoothDevice.type.index,
        'alias': alias,
        'connected': true,
      };
    } catch (e) {
      return null;
    }
  }

  static getActivityCharacteristic(bluetoothDevice, CharacteristicProperties characteristicProperties) async {
    final List<BluetoothService> bluetoothServices = await bluetoothDevice?.discoverServices() ?? <BluetoothService>[];

    BluetoothCharacteristic? characteristic;

    for (var service in bluetoothServices) {
      if (service.isPrimary) {
        for (var bluetoothCharacteristic in service.characteristics) {
          if (characteristicProperties.write && bluetoothCharacteristic.properties.write) {
            characteristic = bluetoothCharacteristic;
            break;
          }
          if (characteristicProperties.read && bluetoothCharacteristic.properties.read) {
            characteristic = bluetoothCharacteristic;
            break;
          }
          if (characteristicProperties.notify && bluetoothCharacteristic.properties.notify) {
            characteristic = bluetoothCharacteristic;
            break;
          }
        }
      }
    }
    if (null == characteristic) return Future.error('没有找到可用的 characteristic');
    return characteristic;
  }

  static whiteByteByAddress(String address, List<int> byteBuffer) async {
    BluetoothDevice? connectBlueDevice = await getDevicesConnectedByAddress(address);
    if (connectBlueDevice == null) {
      return Future.error('蓝牙未连接');
    }

    BluetoothCharacteristic? characteristic = await getActivityCharacteristic(connectBlueDevice, const CharacteristicProperties(write: true));

    if (null == characteristic) return Future.error('没有找到可用的 characteristic');
    debugPrint('byteBuffer ${byteBuffer.length}');

    int mtu = await LocalStore.getInt(address);
    if (mtu == 0) {
      mtu = 20;
    }
    var stepLen = mtu;
    var len = (byteBuffer.length / stepLen).ceil();
    List<List<int>> list = [];
    for(var ll = 0 ; ll < len; ll ++) {
      var startLen = ll * stepLen;
      var endIndex = startLen + stepLen;
      if (endIndex > byteBuffer.length) {
        endIndex = byteBuffer.length;
      }
      list.add(byteBuffer.sublist(startLen, endIndex));
    }

    Future.forEach(list, (List<int> subList) async {
      return await characteristic.write(subList, withoutResponse: false);
    });
  }


  static print(String url, String alias, int pageSize, { Map<String, dynamic>? queryParameters }) async {
    queryParameters ??= {};
    BluetoothDevice bluetoothDevice = await getBluetoothDeviceByAlias(alias);
    Response response = await getHttpInstance().get(url, queryParameters: queryParameters);
    final String content = response.data.toString();
    List<Map<String, dynamic>> contentList = jsonDecode(content);
    final escPosByJson = EscPosByJson(pageSize: pageSize);
    List<int> bytes = await escPosByJson.build(contentList);
    await whiteByteByAddress(bluetoothDevice.id.id, bytes);
  }

  static printTest(String alias) async {
    BluetoothDevice bluetoothDevice = await getBluetoothDeviceByAlias(alias);
    debugPrint('begin pritn test ticket');
    final escPosByJson = EscPosByJson(pageSize: 1);
    try {
      List<int> bytes = await escPosByJson.build(testPrintJson);
      debugPrint('bytes $bytes');
      await whiteByteByAddress(bluetoothDevice.id.id, bytes);
    } catch (e, stacktrace) {
      debugPrint(e.toString());
      debugPrint(stacktrace.toString());
    }


  }
}
